# Honeybee: A Plugin for Environmental Analysis (GPL)
# This file is part of Honeybee.
#
# Copyright (c) 2022, Ladybug Tools.
# You should have received a copy of the GNU Affero General Public License
# along with Honeybee; If not, see <http://www.gnu.org/licenses/>.
# 
# @license AGPL-3.0-or-later <https://spdx.org/licenses/AGPL-3.0-or-later>

"""
Run a point-in-time view-based study for a Honeybee model.
_
Point-in-time view-based recipes require a sky and can output High Dynamic Range
(HDR) images of illuminance, irradiance, luminance or radiance.
_
The `view_count_` input can be used to split each view for parallel processing,
producing multiple images that are recombined into a single .HDR for the view at
the end of the recipe. The recombination process automatically includes an
anti-aliasing pass that smooths and improves the quality of the image. The recipe
also performs an overture calculation prior to splitting each view, which results
in an image with better interpolation between neighboring pixels.

-
    Args:
        _model: A Honeybee Model for which a point-in-time view-based study will be run.
            Note that this model should have views assigned to it in order
            to produce meaningfule results.
        _sky: A Radiance sky from any of the sky components under the "Light Sources" tab.
            Skies can be either CIE, ClimateBased/Custom, or for a specific
            Illuminance/Irradiance. This input can also just be a text definition
            of a sky's paramters. Examples include:
                * cie 21 Mar 9:00 -lat 41.78 -lon -87.75 -tz 5 -type 0
                * climate-based 21 Jun 12:00 -lat 41.78 -lon -87.75 -tz 5 -dni 800 -dhi 120
                * irradiance 0
        _metric_: Either an integer or the full name of a point-in-time metric to be
            computed by the recipe. (Default: luminance). Choose from the following:
                * 0 = illuminance
                * 1 = irradiance
                * 2 = luminance
                * 3 = radiance
        _resolution_: An integer for the maximum dimension of each image in pixels
            (either width or height depending on the input view angle and
            type). (Default: 800).
        view_filter_: Text for a view identifer or a pattern to filter the views of the
            model that are simulated. For instance, `first_floor_*` will simulate
            only the views that have an identifier that starts with `first_floor_`.
            By default, all views in the model will be simulated.
        skip_overture_: A boolean to note whether an ambient file (.amb) should be
            generated for an overture calculation before the view is split
            into smaller views. With an overture calculation, the ambient file
            (aka ambient cache) is first populated with values. Thereby ensuring
            that - when reused to create an image - Radiance uses interpolation
            between already calculated values rather than less reliable
            extrapolation. The overture calculation has comparatively small
            computation time to full rendering but is single-core can become
            time consuming in situations with a high view_count_ and workers.
        radiance_par_: Text for the radiance parameters to be used for ray
            tracing. (Default: -ab 2 -aa 0.25 -ad 512 -ar 16).
        run_settings_: Settings from the "HB Recipe Settings" component that specify
            how the recipe should be run. This can also be a text string of
            recipe settings.
        _run: Set to True to run the recipe and get results. This input can also be
            the integer "2" to run the recipe silently.

    Returns:
        report: Reports, errors, warnings, etc.
        results: High Dynamic Range (HDR) images for each View in the model. These can be
            plugged into the Ladybug "Image Viewer" component to preview the image. 
            They can also be plugged into the "HB False Color" component to convert 
            the image into a false color version. Lastly, it can be connected to 
            the "HB HDR to GIF" component to get a GIF image that is more portable 
            and easily previewed by different software. Pixel values are in the
            standard SI units of the requested input metric.
                * illuminance = lux (aka. lm/m2)
                * irradiance = W/m2
                * luminance = cd/m2 (aka. lm/m2-sr)
                * radiance = W/m2-sr
"""

ghenv.Component.Name = 'HB Point-In-Time View-Based'
ghenv.Component.NickName = 'PITView'
ghenv.Component.Message = '1.4.0'
ghenv.Component.Category = 'HB-Radiance'
ghenv.Component.SubCategory = '3 :: Recipes'
ghenv.Component.AdditionalHelpFromDocStrings = '2'

import os

try:
    from honeybee.model import Model
except ImportError as e:
    raise ImportError('\nFailed to import honeybee:\n\t{}'.format(e))

try:
    from lbt_recipes.recipe import Recipe
except ImportError as e:
    raise ImportError('\nFailed to import lbt_recipes:\n\t{}'.format(e))

try:
    from ladybug_rhino.grasshopper import all_required_inputs, recipe_result, \
        recommended_processor_count
except ImportError as e:
    raise ImportError('\nFailed to import ladybug_rhino:\n\t{}'.format(e))


if all_required_inputs(ghenv.Component) and _run:
    # create the recipe and set the input arguments
    recipe = Recipe('point-in-time-view')
    recipe.input_value_by_name('model', _model)
    recipe.input_value_by_name('sky', _sky)
    recipe.input_value_by_name('metric', _metric_)
    recipe.input_value_by_name('resolution', _resolution_)
    recipe.input_value_by_name('view-filter', view_filter_)
    recipe.input_value_by_name('skip-overture', skip_overture_)
    recipe.input_value_by_name('radiance-parameters', radiance_par_)

    # run the recipe
    silent = True if _run > 1 else False
    project_folder = recipe.run(run_settings_, radiance_check=True, silent=silent)

    # load the results
    try:
        results = recipe_result(recipe.output_value_by_name('results', project_folder))
        if hasattr(results, 'BranchCount') and results.BranchCount == 0:
            raise ValueError()
    except Exception:
        raise Exception(recipe.failure_message(project_folder))



# Honeybee: A Plugin for Environmental Analysis (GPL)
# This file is part of Honeybee.
#
# Copyright (c) 2022, Ladybug Tools.
# You should have received a copy of the GNU Affero General Public License
# along with Honeybee; If not, see <http://www.gnu.org/licenses/>.
# 
# @license AGPL-3.0-or-later <https://spdx.org/licenses/AGPL-3.0-or-later>

"""
Create a Sensor Grid object from radial directions around sensor positions.
_
This type of sensor grid is particularly helpful for studies of multiple view
directions, such as imageless glare studies.
-

    Args:
        _name_: A name for this sensor grid.
        _positions: A list or a datatree of points with one point for the position
            of each radial sensor. Each branch of the datatree will be
            considered as a separate sensor grid.
        _dir_count_: A positive integer for the number of radial directions to be
            generated around each position. (Default: 8).
        _start_vec_: A Vector3D to set the start direction of the generated directions.
            This can be used to orient the resulting sensors to specific parts
            of the scene. It can also change the elevation of the resulting
            directions since this start vector will always be rotated in the
            XY plane to generate the resulting directions. (Default: (0, -1, 0)).
        mesh_radius_: An optional number that can be used to generate a mesh that is
            aligned with the resulting sensors and will automatically be
            assigned to the grid. Such meshes will resemble a circle around
            each sensor with the specified radius and will contain triangular
            faces that can be colored with simulation results. If zero, no mesh
            will be generated for the sensor grid. (Default: 0.2 meters).

    Returns:
        grid: An SensorGrid object that can be used in a grid-based recipe.
        mesh: Analysis mesh that can be passed to the 'Spatial Heatmap' component.
"""

ghenv.Component.Name = 'HB Radial Sensor Grid'
ghenv.Component.NickName = 'SensorGrid'
ghenv.Component.Message = '1.4.0'
ghenv.Component.Category = 'HB-Radiance'
ghenv.Component.SubCategory = '0 :: Basic Properties'
ghenv.Component.AdditionalHelpFromDocStrings = '0'

try:  # import the ladybug_geometry dependencies
    from ladybug_geometry.geometry3d.pointvector import Vector3D, Point3D
except ImportError as e:
    raise ImportError('\nFailed to import ladybug_geometry:\n\t{}'.format(e))

try:  # import the core honeybee dependencies
    from honeybee.typing import clean_and_id_rad_string, clean_rad_string
except ImportError as e:
    raise ImportError('\nFailed to import honeybee:\n\t{}'.format(e))

try:  # import the honeybee-radiance dependencies
    from honeybee_radiance.sensorgrid import SensorGrid
except ImportError as e:
    raise ImportError('\nFailed to import honeybee_radiance:\n\t{}'.format(e))

try:  # import ladybug_rhino dependencies
    from ladybug_rhino.config import conversion_to_meters
    from ladybug_rhino.togeometry import to_vector3d
    from ladybug_rhino.fromgeometry import from_mesh3d, from_point3d, from_vector3d
    from ladybug_rhino.grasshopper import all_required_inputs
except ImportError as e:
    raise ImportError('\nFailed to import ladybug_rhino:\n\t{}'.format(e))


if all_required_inputs(ghenv.Component):
    # set the default name and process the points to tuples
    name = clean_and_id_rad_string('SensorGrid') if _name_ is None else _name_
    pts = [(pt.X, pt.Y, pt.Z) for pt in _positions]
    dir_count = 8 if _dir_count_ is None else _dir_count_
    mesh_radius = 0.2 / conversion_to_meters() if mesh_radius_ is None else mesh_radius_
    try:
        st_vec = to_vector3d(_start_vec_)
    except AttributeError:
        st_vec = Vector3D(0, -1, 0)

    # create the sensor grid object
    id  = clean_rad_string(name) if '/' not in name else clean_rad_string(name.split('/')[0])
    grid = SensorGrid.from_positions_radial(
        id, pts, dir_count=8, start_vector=st_vec, mesh_radius=mesh_radius)

    # set the display name and get outputs
    if _name_ is not None:
        grid.display_name = _name_
    if '/' in name:
        grid.group_identifier = \
            '/'.join(clean_rad_string(key) for key in name.split('/')[1:])
    sensors = grid.sensors
    points = [from_point3d(Point3D(*sen.pos)) for sen in sensors]
    vecs = [from_vector3d(Vector3D(*sen.dir)) for sen in sensors]
    lb_mesh = grid.mesh
    if lb_mesh is not None:
        mesh = from_mesh3d(lb_mesh)



# Honeybee: A Plugin for Environmental Analysis (GPL)
# This file is part of Honeybee.
#
# Copyright (c) 2022, Ladybug Tools.
# You should have received a copy of the GNU Affero General Public License
# along with Honeybee; If not, see <http://www.gnu.org/licenses/>.
# 
# @license AGPL-3.0-or-later <https://spdx.org/licenses/AGPL-3.0-or-later>

"""
Create an mirror radiance modifier from a single reflectance.
-

    Args:
        _name: Text to set the name for the modifier and to be incorporated into
            a unique modifier identifier.
        _r_ref: A number between 0 and 1 for the red reflectance.
        _g_ref: A number between 0 and 1 for the green reflectance.
        _b_ref: A number between 0 and 1 for the blue reflectance.
    
    Returns:
        modifier: An mirror modifier that can be assigned to a Honeybee geometry
            or Modifier Sets.
"""

ghenv.Component.Name = 'HB Mirror Modifier 3'
ghenv.Component.NickName = 'MirrorMod3'
ghenv.Component.Message = '1.4.0'
ghenv.Component.Category = 'HB-Radiance'
ghenv.Component.SubCategory = "1 :: Modifiers"
ghenv.Component.AdditionalHelpFromDocStrings = '0'

try:  # import the core honeybee dependencies
    from honeybee.typing import clean_rad_string, clean_and_id_rad_string
except ImportError as e:
    raise ImportError('\nFailed to import honeybee:\n\t{}'.format(e))

try:  # import the honeybee-radiance dependencies
    from honeybee_radiance.modifier.material import Mirror
except ImportError as e:
    raise ImportError('\nFailed to import honeybee_radiance:\n\t{}'.format(e))

try:  # import ladybug_rhino dependencies
    from ladybug_rhino.grasshopper import all_required_inputs
except ImportError as e:
    raise ImportError('\nFailed to import ladybug_rhino:\n\t{}'.format(e))


if all_required_inputs(ghenv.Component):
    name = clean_and_id_rad_string('MirrorMaterial') if _name_ is None else \
        clean_rad_string(_name_)

    # create the modifier
    modifier = Mirror(name, _r_ref, _g_ref, _b_ref)
    if _name_ is not None:
        modifier.display_name = _name_



# Honeybee: A Plugin for Environmental Analysis (GPL)
# This file is part of Honeybee.
#
# Copyright (c) 2022, Ladybug Tools.
# You should have received a copy of the GNU Affero General Public License
# along with Honeybee; If not, see <http://www.gnu.org/licenses/>.
# 
# @license AGPL-3.0-or-later <https://spdx.org/licenses/AGPL-3.0-or-later>

"""
Calculate Annual Glare Metrics from result (.dgp) files.
_
- Glare Autonmy is a metric describing the percentage of occupied
hours that each sensor is below the glare threshold.
- Spatial Glare Autonomy is a metric describing the percentage of the sensor grid
that is free glare according to the glare threshold and the target time. The sGA
value is expressed as a percentage of the sensors in the analysis grid.

-
    Args:
        _results: An list of annual Radiance result files from the "HB Imageless Annual
            Glare" component (containing the .dgp files and the sun-up-hours.txt).
            This can also be just the path to the folder containing these result
            files.
        _occ_sch_: An annual occupancy schedule as a Ladybug Data Collection or a HB-Energy
            schedule object. This can also be the identifier of a schedule in
            your HB-Energy schedule library. Any value in this schedule that is
            0.1 or above will be considered occupied. If None, a schedule from
            9AM to 5PM on weekdays will be used.
        _glare_thresh_: Threshold for glare autonomy (GA) in DGP (default: 0.4).
        grid_filter_: The name of a grid or a pattern to filter the grids. For instance,
            first_floor_* will simulate only the sensor grids that have an
            identifier that starts with first_floor_. By default all the grids
            will be processed.
        _target_time_: A minimum threshold of occupied time (eg. 95% of the time), above
            which a given sensor passes and contributes to the spatial glare
            autonomy. (Default: 95%).

    Returns:
        report: Reports, errors, warnings, etc.
        GA: Glare autonomy results in percent. GA is the percentage of occupied hours
            that each sensor is free of glare according to the glare threshold.
            Each value is for a different sensor of the grid. These can be plugged
            into the "LB Spatial Heatmap" component along with meshes of the sensor
            grids to visualize results.
        sGA: Spatial glare autonomy as a percentage of the sensors for each analysis grid
            that does not exceed the glare threshold for a specified fraction of
            occupied hours, ie. the target time.
        pass_fail: A data tree of zeros and ones, which indicate whether a given sensor
            passes the criteria for being free of glare (1) or fails the criteria (0).
            Being free of glare does not necessarily mean that the sensor is glare-free
            for all hours, but that it is glare-free for a minimum percentage of
            occupied hours defined by the target time. Each value is for a different
            sensor of the grid. These can be plugged into the "LB Spatial Heatmap"
            component along with meshes of the sensor grids to visualize results.
"""

ghenv.Component.Name = "HB Annual Glare Metrics"
ghenv.Component.NickName = 'GlareMetrics'
ghenv.Component.Message = '1.4.0'
ghenv.Component.Category = 'HB-Radiance'
ghenv.Component.SubCategory = '4 :: Results'
ghenv.Component.AdditionalHelpFromDocStrings = '1'

import os

try:
    from ladybug.datacollection import BaseCollection
except ImportError as e:
    raise ImportError('\nFailed to import ladybug:\n\t{}'.format(e))

try:
    from honeybee_radiance.postprocess.annualglare import glare_autonomy_from_folder
except ImportError as e:
    raise ImportError('\nFailed to import honeybee_radiance:\n\t{}'.format(e))

try:
    from honeybee_energy.lib.schedules import schedule_by_identifier
except ImportError as e:  # honeybee schedule library is not available
    schedule_by_identifier = None

try:
    from ladybug_rhino.grasshopper import all_required_inputs, list_to_data_tree, \
        data_tree_to_list
except ImportError as e:
    raise ImportError('\nFailed to import ladybug_rhino:\n\t{}'.format(e))


if all_required_inputs(ghenv.Component):
    # set default values for the thresholds and the grid filter
    grid_filter_ = '*' if grid_filter_ is None else grid_filter_
    _glare_thresh_ = _glare_thresh_ if _glare_thresh_ else 0.4

    # process the schedule
    if _occ_sch_ is None:
        schedule = None
    elif isinstance(_occ_sch_, BaseCollection):
        schedule = _occ_sch_.values
    elif isinstance(_occ_sch_, str):
        if schedule_by_identifier is not None:
            schedule = schedule_by_identifier(_occ_sch_).values()
        else:
            raise ValueError('honeybee-energy must be installed to reference '
                             'occupancy schedules by identifier.')
    else:  # assume that it is a honeybee schedule object
        try:
            schedule = _occ_sch_.values()
        except TypeError:  # it's probably a ScheduleFixedInterval
            schedule = _occ_sch_.values
    if schedule is not None:
        bin_schedule = []
        for val in schedule:
            bin_val = 1 if val >= 0.1 else 0
            bin_schedule.append(bin_val)
        schedule = bin_schedule

    # compute the annual metrics
    res_folder = os.path.dirname(_results[0]) if os.path.isfile(_results[0]) \
        else _results[0]
    GA = glare_autonomy_from_folder(
        res_folder, schedule, _threshold_, grid_filter_)
    GA = list_to_data_tree(GA)

    # process the input values into a readable format
    ga_mtx = [item[-1] for item in data_tree_to_list(GA)]
    _target_time_ = 95 if _target_time_ is None else _target_time_

    # determine whether each point passes or fails
    pass_fail = [[int(val > _target_time_) for val in grid] for grid in ga_mtx]

    # compute spatial glare autonomy from the pass/fail results
    sGA = [sum(pf_list) / len(pf_list) for pf_list in pass_fail]

    pass_fail = list_to_data_tree(pass_fail)  # convert matrix to data tree



# Honeybee: A Plugin for Environmental Analysis (GPL)
# This file is part of Honeybee.
#
# Copyright (c) 2022, Ladybug Tools.
# You should have received a copy of the GNU Affero General Public License
# along with Honeybee; If not, see <http://www.gnu.org/licenses/>.
# 
# @license AGPL-3.0-or-later <https://spdx.org/licenses/AGPL-3.0-or-later>

"""
Calculate Annual Daylight Metrics from a result (.ill) files.

-
    Args:
        _results: An list of annual Radiance result files from the "HB Annual Daylight"
            component (containing the .ill files and the sun-up-hours.txt).
            This can also be just the path to the folder containing these
            result files.
        _occ_sch_: An annual occupancy schedule as a Ladybug Data Collection or a HB-Energy
            schedule object. This can also be the identifier of a schedule in
            your HB-Energy schedule library. Any value in this schedule that is
            0.1 or above will be considered occupied. If None, a schedule from
            9AM to 5PM on weekdays will be used.
        _threshold_: Threshhold for daylight autonomy (DA) in lux (default: 300).
        _min_max_: A list for min, max illuminacne thresholds for useful daylight illuminance
            in lux. (Default: (100, 3000)).
        grid_filter_: The name of a grid or a pattern to filter the grids. For instance,
            first_floor_* will simulate only the sensor grids that have an
            identifier that starts with first_floor_. By default all the grids
            will be processed.

    Returns:
        report: Reports, errors, warnings, etc.
        DA: Daylight autonomy results in percent. DA is the percentage of occupied hours
            that each sensor recieves equal or more than the illuminance threshold.
            Each value is for a different sensor of the grid. These can be plugged
            into the "LB Spatial Heatmap" component along with meshes of the sensor
            grids to visualize results. These can also be connected to the "HB
            Spatial Daylight Autonomy" component to compute spatial daylight
            autonomy for each grid.
        cDA: Continuous daylight autonomy results in percent. cDA is similar to DA except
            that values below the illuminance threshold can still count partially
            towards the final percentage. Each value is for a different
            sensor of the grid. These can be plugged into the "LB Spatial Heatmap"
            component along with meshes of the sensor grids to visualize results.
        UDI: Useful daylight illuminance results in percent. UDI is the percentage of time
            that illuminace falls between minimum and maximum thresholds. Each value
            is for a different sensor of the grid. These can be plugged into the
            "LB Spatial Heatmap" component along with meshes of the sensor grids
            to visualize results.
        UDI_low: Results for the percent of time that is below the lower threshold
            of useful daylight illuminance in percent. Each value is for a different
            sensor of the grid. These can be plugged into the "LB Spatial Heatmap"
            component along with meshes of the sensor grids to visualize results.
        UDI_up: Results for the percent of time that is above the upper threshold
            of useful daylight illuminance in percent. Each value is for a different
            sensor of the grid. These can be plugged into the "LB Spatial Heatmap"
            component along with meshes of the sensor grids to visualize results.
"""

ghenv.Component.Name = "HB Annual Daylight Metrics"
ghenv.Component.NickName = 'AnnualMetrics'
ghenv.Component.Message = '1.4.0'
ghenv.Component.Category = 'HB-Radiance'
ghenv.Component.SubCategory = '4 :: Results'
ghenv.Component.AdditionalHelpFromDocStrings = '1'

import os

try:
    from ladybug.datacollection import BaseCollection
except ImportError as e:
    raise ImportError('\nFailed to import ladybug:\n\t{}'.format(e))

try:
    from honeybee_radiance.postprocess.annualdaylight import metrics_from_folder
except ImportError as e:
    raise ImportError('\nFailed to import honeybee_radiance:\n\t{}'.format(e))

try:
    from honeybee_energy.lib.schedules import schedule_by_identifier
except ImportError as e:  # honeybee schedule library is not available
    schedule_by_identifier = None

try:
    from ladybug_rhino.grasshopper import all_required_inputs, list_to_data_tree
except ImportError as e:
    raise ImportError('\nFailed to import ladybug_rhino:\n\t{}'.format(e))


if all_required_inputs(ghenv.Component):
    # set default values for the thresholds and the grid filter
    grid_filter_ = '*' if grid_filter_ is None else grid_filter_
    _threshold_ = _threshold_ if _threshold_ else 300
    if len(_min_max_) != 0:
        assert len(_min_max_), 'Expected two values for _min_max_.'
        min_t = _min_max_[0]
        max_t = _min_max_[1]
    else:
        min_t = 100
        max_t = 3000

    # process the schedule
    if _occ_sch_ is None:
        schedule = None
    elif isinstance(_occ_sch_, BaseCollection):
        schedule = _occ_sch_.values
    elif isinstance(_occ_sch_, str):
        if schedule_by_identifier is not None:
            schedule = schedule_by_identifier(_occ_sch_).values()
        else:
            raise ValueError('honeybee-energy must be installed to reference '
                             'occupancy schedules by identifier.')
    else:  # assume that it is a honeybee schedule object
        try:
            schedule = _occ_sch_.values()
        except TypeError:  # it's probably a ScheduleFixedInterval
            schedule = _occ_sch_.values
    if schedule is not None:
        bin_schedule = []
        for val in schedule:
            bin_val = 1 if val >= 0.1 else 0
            bin_schedule.append(bin_val)
        schedule = bin_schedule

    # compute the annual metrics
    res_folder = os.path.dirname(_results[0]) if os.path.isfile(_results[0]) \
        else _results[0]
    DA, cDA, UDI_low, UDI, UDI_up = metrics_from_folder(
        res_folder, schedule, _threshold_, min_t, max_t, grid_filter_)
    DA = list_to_data_tree(DA)
    cDA = list_to_data_tree(cDA)
    UDI = list_to_data_tree(UDI)
    UDI_low = list_to_data_tree(UDI_low)
    UDI_up = list_to_data_tree(UDI_up)


# This file is part of Honeybee.
#
# Copyright (c) 2022, Ladybug Tools.
# You should have received a copy of the GNU Affero General Public License
# along with Honeybee; If not, see <http://www.gnu.org/licenses/>.
# 
# @license AGPL-3.0-or-later <https://spdx.org/licenses/AGPL-3.0-or-later>

"""
Import the hourly illuminance or irradiance results of an annual daylight or irradiance
study to a list of data collections.
_
The resulting data collections can be visulized using the ladybug components or
deconstructed for detailed analysis with native Grasshopper math components.

-
    Args:
        _results: An list of annual Radiance result files from the "HB Annual Daylight"
            component (containing the .ill files and the sun-up-hours.txt).
            This can also be just the path to the folder containing these
            result files.
        sel_pts_: An optional point or list of points, which will be used to filter
            the sensors for which data collections will be imported. If there
            is an input here, the all_pts_ must be connected.
        all_pts_: The data tree of all sensor points that were used in the simulation.
            This is required in order to look up the index of the sel_pts_ in
            the results matrices. 

 Returns:
        report: Reports, errors, warnings, etc.
        data: A list of hourly data collections containing illuminance or irradiance results.
            These can be visulized using the ladybug components or deconstructed
            for detailed analysis with native Grasshopper math components.
"""

ghenv.Component.Name = 'HB Annual Results to Data'
ghenv.Component.NickName = 'AnnualToData'
ghenv.Component.Message = '1.4.1'
ghenv.Component.Category = 'HB-Radiance'
ghenv.Component.SubCategory = '4 :: Results'
ghenv.Component.AdditionalHelpFromDocStrings = '2'

import os

try:
    from ladybug.datatype.illuminance import Illuminance
    from ladybug.datatype.energyflux import Irradiance
    from ladybug.datatype.time import Time
    from ladybug.datatype.fraction import Fraction
    from ladybug.analysisperiod import AnalysisPeriod
    from ladybug.header import Header
    from ladybug.datacollection import HourlyContinuousCollection
except ImportError as e:
    raise ImportError('\nFailed to import ladybug:\n\t{}'.format(e))

try:
    from honeybee_radiance.postprocess.annualdaylight import _process_input_folder
except ImportError as e:
    raise ImportError('\nFailed to import honeybee_radiance:\n\t{}'.format(e))

try:
    from ladybug_rhino.config import tolerance
    from ladybug_rhino.togeometry import to_point3d
    from ladybug_rhino.grasshopper import all_required_inputs, list_to_data_tree, \
        data_tree_to_list
except ImportError as e:
    raise ImportError('\nFailed to import ladybug_rhino:\n\t{}'.format(e))


def file_to_data(ill_file, point_filter, su_pattern, header, timestep):
    """Get a list of data collections for a given result file."""
    data_colls = []
    with open(ill_file) as results:
        if point_filter is None:
            for pt_res in results:
                base_values = [0] * 8760 * timestep
                for val, hr in zip(pt_res.split(), su_pattern):
                    base_values[hr] = float(val)
                data_colls.append(HourlyContinuousCollection(header, base_values))
        else:
            for i, pt_res in enumerate(results):
                if i in point_filter:
                    base_values = [0] * 8760 * timestep
                    for val, hr in zip(pt_res.split(), su_pattern):
                        base_values[hr] = float(val)
                    data_colls.append(HourlyContinuousCollection(header, base_values))
    return data_colls


def find_point_in_grid(s_pt, all_pts):
    """Find the index of a point in a list of list of grids."""
    m_pts = []
    for i, grid_pts in enumerate(all_pts):
        for j, g_pt in enumerate(grid_pts):
            if g_pt.is_equivalent(s_pt, tolerance):
                m_pts.append((i, j))
    return m_pts


if all_required_inputs(ghenv.Component):
    # get the relevant .ill files
    res_folder = os.path.dirname(_results[0]) if os.path.isfile(_results[0]) \
        else _results[0]
    grids, sun_up_hours = _process_input_folder(res_folder, '*')

    # check the sel_pts and all_pts input
    pt_filter = [None for i in grids]
    if len(sel_pts_) != 0:
        pt_filter = [[] for i in grids]
        all_pts = [[to_point3d(pt) for pt in dat[-1]] for dat in data_tree_to_list(all_pts_)]
        assert len(all_pts) != 0, 'all_pts_ must be connected in order to use sel_pts_.'
        sel_pts = [to_point3d(pt) for pt in sel_pts_]
        for s_pt in sel_pts:
            m_pts = find_point_in_grid(s_pt, all_pts)
            for i, j in m_pts:
                pt_filter[i].append(j)

    # extract the timestep if it exists
    timestep, has_t_step = 1, False
    tstep_file = os.path.join(res_folder, 'timestep.txt')
    if os.path.isfile(tstep_file):  # it's an annual irradiance simulation
        with open(tstep_file) as tf:
            timestep = int(tf.readline())
        has_t_step = True

    # parse the sun-up-hours
    sun_up_hours = [int(h * timestep) for h in sun_up_hours]

    # create the header that will be used for all of the data collections
    aper = AnalysisPeriod(timestep=timestep)
    if 'direct_sun_hours' in res_folder:
        head = Header(Time(), 'hr', aper)
    elif has_t_step:
        head = Header(Irradiance(), 'W/m2', aper)
    else:
        head = Header(Illuminance(), 'lux', aper)
    dgp_head = Header(Fraction(), 'fraction', aper, metadata={'type': 'Daylight Glare Probability (DGP)'})

    # create the data collections from the .ill files
    data = []
    for grid_info, p_filt in zip(grids, pt_filter):
        ill_file = os.path.join(res_folder, '%s.ill' % grid_info['full_id'])
        dgp_file = os.path.join(res_folder, '%s.dgp' % grid_info['full_id'])
        if os.path.isfile(dgp_file):
            data_list = file_to_data(dgp_file, p_filt, sun_up_hours, dgp_head, timestep)
        else:
            data_list = file_to_data(ill_file, p_filt, sun_up_hours, head, timestep)
        data.append(data_list)
    data = list_to_data_tree(data)
